---
title: prometheus
type: metrics
status: stable
---

<!--
     THIS FILE IS AUTOGENERATED!

     To make changes please edit the contents of:
     lib/metrics/prometheus.go
-->

import Tabs from '@theme/Tabs';
import TabItem from '@theme/TabItem';


Host endpoints (`/metrics` and `/stats`) for Prometheus scraping.


<Tabs defaultValue="common" values={[
  { label: 'Common', value: 'common', },
  { label: 'Advanced', value: 'advanced', },
]}>

<TabItem value="common">

```yaml
# Common config fields, showing default values
metrics:
  prometheus:
    prefix: benthos
    path_mapping: ""
```

</TabItem>
<TabItem value="advanced">

```yaml
# All config fields, showing default values
metrics:
  prometheus:
    prefix: benthos
    path_mapping: ""
    push_url: ""
    push_interval: ""
    push_job_name: benthos_push
    push_basic_auth:
      username: ""
      password: ""
```

</TabItem>
</Tabs>

Metrics paths will differ from [the standard list](/docs/components/metrics/about#metric_names) in order to comply with Prometheus naming restrictions, where dots are replaced with underscores (and underscores replaced with double underscores). This change is made _before_ the mapping from `path_mapping` is applied.

## Fields

### `prefix`

A string prefix to add to all metrics.


Type: `string`  
Default: `"benthos"`  

### `path_mapping`

An optional [Bloblang mapping](/docs/guides/bloblang/about) that allows you to rename or prevent certain metrics paths from being exported. When metric paths are created, renamed and dropped a trace log is written, enabling TRACE level logging is therefore a good way to diagnose path mappings. BETA FEATURE: Labels can also be created for the metric path by mapping meta fields.


Type: `string`  
Default: `""`  

```yaml
# Examples

path_mapping: this.replace("input", "source").replace("output", "sink")

path_mapping: |-
  if ![
    "benthos_input_received",
    "benthos_input_latency",
    "benthos_output_sent"
  ].contains(this) { deleted() }

path_mapping: |-
  let matches = this.re_find_all_submatch("resource_processor_([a-zA-Z]+)_(.*)")
  meta processor = $matches.0.1 | deleted()
  root = $matches.0.2 | deleted()
```

### `push_url`

An optional [Push Gateway URL](#push-gateway) to push metrics to.


Type: `string`  
Default: `""`  

### `push_interval`

The period of time between each push when sending metrics to a Push Gateway.


Type: `string`  
Default: `""`  

### `push_job_name`

An identifier for push jobs.


Type: `string`  
Default: `"benthos_push"`  

### `push_basic_auth`

The Basic Authentication credentials.


Type: `object`  

### `push_basic_auth.username`

The Basic Authentication username.


Type: `string`  
Default: `""`  

### `push_basic_auth.password`

The Basic Authentication password.


Type: `string`  
Default: `""`  

## Push Gateway

The field `push_url` is optional and when set will trigger a push of
metrics to a [Prometheus Push Gateway](https://prometheus.io/docs/instrumenting/pushing/)
once Benthos shuts down. It is also possible to specify a
`push_interval` which results in periodic pushes.

The Push Gateway is useful for when Benthos instances are short lived. Do not
include the "/metrics/jobs/..." path in the push URL.

If the Push Gateway requires HTTP Basic Authentication it can be configured with
`push_basic_auth`.

